package ru.dbotthepony.mc.otm.recipe

import net.minecraft.world.item.crafting.CraftingInput
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.item.crafting.ShapedRecipePattern
import ru.dbotthepony.mc.otm.core.get
import ru.dbotthepony.mc.otm.core.isNotEmpty

val ShapedRecipePattern.isIncomplete: Boolean
	get() = ingredients().any { it.items.size == 1 && it.hasNoItems() }

val ShapedRecipePattern.width: Int get() = width()
val ShapedRecipePattern.height: Int get() = height()

operator fun ShapedRecipePattern.get(column: Int, row: Int): Ingredient {
	return ingredients()[column + row * width]
}

operator fun ShapedRecipePattern.get(column: Int, row: Int, flop: Boolean): Ingredient {
	return if (flop)
		get(width - column - 1, row)
	else
		get(column, row)
}

fun ShapedRecipePattern.preemptiveTest(t: CraftingInput, fromColumn: Int, fromRow: Int, flop: Boolean): Boolean {
	if (t.width() - fromColumn < width || t.height() - fromRow < height)
		return false

	for (column in 0 until width) {
		for (row in 0 until height) {
			val item = t[fromColumn + column, fromRow + row, flop]
			val ingredient = this[column, row, flop]

			if (!ingredient.test(item) && item.isNotEmpty) {
				return false
			}
		}
	}

	return true
}

fun ShapedRecipePattern.preemptiveTest(t: CraftingInput): Boolean {
	if (t.width() < width || t.height() < height)
		return false

	for (column in 0 .. t.width() - width)
		for (row in 0 .. t.height() - height)
			if (preemptiveTest(t, column, row, false) || preemptiveTest(t, column, row, true))
				return true

	return false
}

fun ShapedRecipePattern.test(t: CraftingInput, fromColumn: Int, fromRow: Int, flop: Boolean): Boolean {
	if (t.width() - fromColumn < width || t.height() - fromRow < height)
		return false

	for (column in 0 until width)
		for (row in 0 until height)
			if (!this[column, row, flop].test(t[fromColumn + column, fromRow + row, flop]))
				return false

	return true
}

fun ShapedRecipePattern.test(t: CraftingInput): Boolean {
	if (t.width() < width || t.height() < height)
		return false

	for (column in 0 .. t.width() - width)
		for (row in 0 .. t.height() - height)
			if (test(t, column, row, false) || test(t, column, row, true))
				return true

	return false
}
